/*
	CVSNT Generic API
    Copyright (C) 2004 Tony Hoyle and March-Hare Software Ltd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef FILEENTRY__H
#define FILEENTRY__H

#include <map>

#include "cvs_string.h"
#include "cvs_smartptr.h"

struct CvsBasicEntry
{
	CvsBasicEntry() { ignored=false; }
	virtual ~CvsBasicEntry() { }
	cvs::filename logical_name; // Name as seen by user
	cvs::filename physical_name; // Relative path to repository
	cvs::filename module_name; // Module source name

	cvs::string tag;
	cvs::string date;
	cvs::string version;

	bool ignored; // Used for modules
};

struct CvsDirectoryEntry : public CvsBasicEntry
{
	typedef std::map<cvs::filename,cvs::smartptr<CvsDirectoryEntry*> > children_t;
	virtual ~CvsDirectoryEntry() { }

	children_t children;
};

struct CvsFileEntry : public CvsBasicEntry
{
	virtual ~CvsFileEntry() { }

	cvs::string expansion_options;
	cvs::string merge_tag1;
	cvs::string merge_tag2;
	time_t commit_date;
	cvs::string edit_revision;
	cvs::string edit_tag;
	cvs::string bugid;
};

#endif

