/*
	CVSNT Generic API
    Copyright (C) 2004 Tony Hoyle and March-Hare Software Ltd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <config.h>
#include "lib/api_system.h"
#include <assert.h>
#include <sys/types.h>
#include <stdlib.h>
#include <ctype.h>
#include "lib/getdate.h"
#include "TagDate.h"

CTagDate::CTagDate(bool range)
{
	m_range = range;
}

CTagDate::~CTagDate()
{
}

bool CTagDate::AddDate(const char *date)
{
	return AddGenericTag(true,date);
}

bool CTagDate::AddTag(const char *tag)
{
	return AddGenericTag(false,tag);
}

/* Add a tag or date 
 * Supports ranges as:
 *
 * tag (just tag)
 * tag1:tag2  (inclusive)
 * tag1::tag2 (exclusive)
 * tag1:::tag2 (exclude tag1, include tag2)
 * <tag (before tag)
 * <=tag (tag and before tag)
 * >tag (after tag)
 * >=tag (tag and after tag)
 */
bool CTagDate::AddGenericTag(bool isDate, const char *tag)
{
	CTagDateItem item;

	assert(tag && *tag);

	if(strchr(tag,':'))
	{
		if(!m_range)
			return false; /* Attempted to specify range on non-ranged tag */

		const char *p = strchr(tag,':');
		int type=0;
		std::string tag1,tag2;
		
		tag1.assign(tag,tag-p);
		while(*p==':')
		{
			p++;
			type++;
		}
		tag2.assign(p);

		if(!isValidTag(isDate,tag1.c_str(),item.m_date))
			return false;

		if(!isValidTag(isDate,tag2.c_str(),item.m_date))
			return false;

		item.m_isDate = isDate;
		item.m_text = tag1;
		if(type==1)
			item.m_tagRangeType = trtRangeStartIncluded;
		else
			item.m_tagRangeType = trtRangeStartExcluded;
		m_list.push_back(item);

		item.m_isDate = isDate;
		item.m_text = tag2;
		if(type==1 || type==3)
			item.m_tagRangeType = trtRangeEndIncluded;
		else
			item.m_tagRangeType = trtRangeEndExcluded;
		m_list.push_back(item);
	}
	else
	{
		if(m_range)
		{
			if(tag[0]=='<' && tag[1]=='=')
			{ 
				item.m_tagRangeType=trtLessThanOrEqual;
				tag+=2;
			}
			else if(tag[0]=='<')
			{
				item.m_tagRangeType=trtLessThan;
				tag++;
			}
			else if(tag[0]=='>' && tag[1]=='=')
			{
				item.m_tagRangeType=trtGreaterThanOrEqual;
				tag+=2;
			}
			else if(tag[0]=='>')
			{
				item.m_tagRangeType=trtGreaterThan;
				tag++;
			}
			else
				item.m_tagRangeType=trtSingle;
		}
		else
			item.m_tagRangeType=trtSingle;

		if(!isValidTag(isDate,tag,item.m_date))
			return false;

		item.m_isDate = isDate;
		item.m_text = tag;
		m_list.push_back(item);
	}
	return true;
}

/* Deterimine if a tag is syntactically valid
 * supports (for non-dates):
 *
 * 1.2.3.4 (numeric)
 * foo (text)
 * foo.3 (specific revision on branch)
 * @43346363AA234 (commit identifier)
 * 
 * for dates the generic date parser is used.
 */
bool CTagDate::isValidTag(bool isDate, const char *tag, time_t& time)
{
	const char *p = tag;
	if(!isDate)
	{
		if(isdigit(*p))
		{
			for(;*p;p++)
				if(!isdigit(*p) && *p!='.')
					break;
			if(*p)
				return false;
			return true;
		}
		if(*p=='@')
			return true;

		for(;*p;p++)
			if(!isalnum(*p))
				break;
		if(*p && *p!='.')
			return false;
		p++;
		if(*p)
		{
			for(;*p;p++)
				if(!isdigit(*p))
					break;
			if(*p)
				return false;
		}
		return true;
	}
	else
	{
		time = get_date((char*)tag,NULL);
		if(time==(time_t)-1)
			return false;
		return true;
	}
}
