/*
	CVSNT Generic API
    Copyright (C) 2004 Tony Hoyle and March-Hare Software Ltd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef TAGDATE__H
#define TAGDATE__H

#include <string>
#include <vector>

enum tagRangeType_t
{
	trtSingle,
	trtRangeStartIncluded,
	trtRangeStartExcluded,
	trtRangeEndIncluded,
	trtRangeEndExcluded,
	trtLessThan,
	trtLessThanOrEqual,
	trtGreaterThan,
	trtGreaterThanOrEqual
};

class CTagDateItem
{
	friend class CTagDate;
public:
	CVSAPI_EXPORT virtual ~CTagDateItem() { }
	CVSAPI_EXPORT CTagDateItem(const CTagDateItem& other)
	{
		m_isDate = other.m_isDate;
		m_tagRangeType = other.m_tagRangeType;
		m_text = other.m_text;
		m_date = other.m_date;
	}

	CVSAPI_EXPORT bool isDate() const { return m_isDate; };
	CVSAPI_EXPORT bool isBranchOrTag() const { return !m_isDate; }
	CVSAPI_EXPORT tagRangeType_t getType() const { return m_tagRangeType; }
	CVSAPI_EXPORT const char *text() const { return m_text.c_str(); }
	CVSAPI_EXPORT const time_t date() const { assert(m_isDate); return m_date; }

protected:
	CTagDateItem() { }

	bool m_isDate;
	tagRangeType_t m_tagRangeType;
	std::string m_text;
	time_t m_date;
};

class CTagDate
{
public:
	CVSAPI_EXPORT CTagDate(bool range);
	CVSAPI_EXPORT virtual ~CTagDate();

	CVSAPI_EXPORT size_t size() { return m_list.size(); }
	CVSAPI_EXPORT const CTagDateItem& operator[](size_t item) { return m_list[item]; }

	CVSAPI_EXPORT bool AddTag(const char *tag);
	CVSAPI_EXPORT bool AddDate(const char *date);

protected:
	bool m_range;
	std::vector<CTagDateItem> m_list;

	bool AddGenericTag(bool isDate, const char *tag);
	bool isValidTag(bool isDate, const char *tag, time_t& time);
};

#endif
