/*
    CVSNT Generic API
    Copyright (C) 2004 Tony Hoyle and March-Hare Software Ltd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef API_SYSTEM__H
#define API_SYSTEM__H

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
/* Win32 is both username and filename case insensitive, with multiple path separators */
#define IsSepeparator(c) ((c)=='\\' || (c)=='/')
#define FsCaseSensitive __case_sensitive
#define CVS_CASEFOLD (FsCaseSensitive?0:FNM_CASEFOLD)
#define CompareFileChar(c,d) __cfc(c,d,FsCaseSensitive)
#define CompareUserChar(c,d) __cfc(c,d,0)
#define fncmp __fncmp
#define fnncmp __fnncmp
#define usercmp stricmp
#define userncmp strnicmp

/* Make Win32 use 64bit times, and fix a VC compatibility bug */
#include <time.h>
#define time_t __time64_t
#define gmtime _gmtime64
#define localtime _localtime64
#define mktime _mktime64
#define time _time64
#define ctime wnt_ctime
#define asctime wnt_asctime

#define vsnprintf _vsnprintf
#define snprintf _snprintf
#define strcasecmp stricmp
#define strncasecmp strnicmp

#ifndef CVSAPI_EXPORT
#define CVSAPI_EXPORT __declspec(dllimport)

#ifndef PATH_MAX
#define PATH_MAX _MAX_PATH
#endif

#define CRLF_DEFAULT ltCrLf

#endif

CVSAPI_EXPORT extern int __case_sensitive;

CVSAPI_EXPORT char *wnt_asctime(const struct tm *tm);
CVSAPI_EXPORT char *wnt_ctime(const time_t *t);

#endif

#ifdef __APPLE__
/* Mac OSX can have multiple concepts of case insensitivity, but we just
   define as insensitive here (HFS default).  It would be relatively
   trivial to replace this with a routine to detect it dynamically */
#define FsCaseSensitive 0
#define CVS_CASEFOLD FNM_CASEFOLD
#define CompareFileChar(c,d) (((c)==(d))?0:tolower(c)-tolower(d))
#define fncmp strcasecmp
#define fnncmp strncasecmp
#endif

#ifdef __OS400__
/* AS400 is case insensitive.. sort of... */
#define FsCaseSensitive 0
#define CVS_CASEFOLD FNM_CASEFOLD
#define CompareFileChar(c,d) (((c)==(d))?0:tolower(c)-tolower(d))
#define fncmp strcasecmp
#define fnncmp strncasecmp
#endif

#ifndef IsSeparator
#define IsSeparator(c) ((c)=='/')
#endif

#ifndef FsCaseSensitive
#define FsCaseSensitive 1
#endif

#ifndef CVS_CASEFOLD
#define CVS_CASEFOLD 0
#endif

#ifndef CompareFileChar
#define CompareFileChar(c,d) ((c)-(d))
#endif

#ifndef CompareUserChar
#define CompareUserChar(c,d) ((c)-(d))
#endif

#ifndef fncmp
#define fncmp strcmp
#endif

#ifndef fnncmp
#define fnncmp strncmp
#endif

#ifndef usercmp
#define usercmp strcmp
#endif

#ifndef userncmp
#define userncmp strncmp
#endif

#ifndef CVSAPI_EXPORT
#define CVSAPI_EXPORT
#endif

/* Does platform use CRLF by default.  To be replaced by a more generic mechanism once the
   core is rewritten */
#ifndef CRLF_DEFAULT
#define CRLF_DEFAULT ltLf
#endif

/* Some platforms (HPUX!) use a non-standard name for the character set */
#ifndef UTF8_CHARSET
#define UTF8_CHARSET "UTF-8"
#endif

#include "fncmp.h"

#ifdef __cplusplus
}
#endif

#endif
