#ifdef INC_MYSQL
/*
	CVSNT Generic API
    Copyright (C) 2004 Tony Hoyle and March-Hare Software Ltd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winsock.h>
#define vsnprintf _vsnprintf
#endif
#include <mysql.h>
#include <string>
#include "MySqlRecordset.h"
#include "MySqlConnection.h"
#include "MySqlRecordset.h"

CMySqlConnection::CMySqlConnection()
{
}

CMySqlConnection::~CMySqlConnection()
{
	Close();
}

bool CMySqlConnection::Open(const char *host, const char *database, const char *username, const char *password)
{
	MYSQL *mysql =  mysql_init(NULL);
	if(!mysql)
		return false;

	m_connect = mysql_real_connect(mysql,host,username,password,database,0,NULL,0);
	if(!m_connect)
		return false;

	return true;
}

bool CMySqlConnection::Close()
{
	if(m_connect)
		mysql_close(m_connect);
	m_connect = NULL;
	return true;
}

CSqlRecordset& CMySqlConnection::Execute(const char *string, ...)
{
	int len;

	// this would be easy with cvs::sprintf :)
	std::string str;
	va_list va;
	va_start(va,string);
	str.resize(strlen(string)+1);
	do
	{
		len = vsnprintf((char*)str.data(),str.size(),string,va);
		if(len==-1 || len>=(int)str.size())
			str.resize(len==-1?str.size()*2:len+1);
	} while(len==-1 || len>=(int)str.size());
	va_end(va);
	str.resize(len);

	size_t nRs;
	for(nRs=0; nRs<m_rsList.size(); nRs++)
		if(m_rsList[nRs].Closed())
			break;
	if(nRs==m_rsList.size())
		m_rsList.resize(nRs+1);

	CMySqlRecordset& rs = m_rsList[nRs];

	/* Unfortunately the mysql binary API changes with each release... I have 4.1 */
	if((rs.m_stmt=mysql_stmt_init(m_connect))==0)
		return rs;
	if(mysql_stmt_prepare(rs.m_stmt,str.c_str(),(unsigned long)str.length()))
		return rs;
	if(mysql_stmt_execute(rs.m_stmt))
		return rs;
	
	if(!rs.Init())
		return rs;

	return rs;
}

#endif