#ifdef INC_MYSQL
/*
	CVSNT Generic API
    Copyright (C) 2004 Tony Hoyle and March-Hare Software Ltd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winsock.h>
#define vsnprintf _vsnprintf
#define snprintf _snprintf
#endif
#include <mysql.h>
#include <string>
#include "MySqlRecordset.h"

CMySqlField::CMySqlField()
{
	bind=NULL;
	buffer=NULL;
}

CMySqlField::~CMySqlField()
{
	delete[] buffer;
}

CMySqlField::operator int()
{
	switch(bind->buffer_type)
	{
		case MYSQL_TYPE_TINY:
			return *(char*)buffer;
		case MYSQL_TYPE_SHORT:
			return *(short *)buffer;
		case MYSQL_TYPE_LONG:
			return *(long *)buffer;
		case MYSQL_TYPE_LONGLONG:
			return (int)*(long long *)buffer;
		case MYSQL_TYPE_FLOAT:
			return (int)*(float*)buffer;
		case MYSQL_TYPE_DOUBLE:
			return (int)*(double*)buffer;
		case MYSQL_TYPE_TIME:
		case MYSQL_TYPE_DATE:
		case MYSQL_TYPE_DATETIME:
		case MYSQL_TYPE_TIMESTAMP:
			return 0;
		case MYSQL_TYPE_STRING:
		case MYSQL_TYPE_VAR_STRING:
			return atoi((const char*)buffer);
		case MYSQL_TYPE_TINY_BLOB:
		case MYSQL_TYPE_BLOB:
		case MYSQL_TYPE_MEDIUM_BLOB:
		case MYSQL_TYPE_LONG_BLOB:
			return 0;
		default:
			return 0;
	}
}

CMySqlField::operator size_t()
{
	switch(bind->buffer_type)
	{
		case MYSQL_TYPE_TINY:
			return *(char*)buffer;
		case MYSQL_TYPE_SHORT:
			return *(short *)buffer;
		case MYSQL_TYPE_LONG:
			return *(int *)buffer;
		case MYSQL_TYPE_LONGLONG:
			return (size_t)*(long long *)buffer;
		case MYSQL_TYPE_FLOAT:
			return (size_t)*(float*)buffer;
		case MYSQL_TYPE_DOUBLE:
			return (size_t)*(double*)buffer;
		case MYSQL_TYPE_TIME:
		case MYSQL_TYPE_DATE:
		case MYSQL_TYPE_DATETIME:
		case MYSQL_TYPE_TIMESTAMP:
			return 0;
		case MYSQL_TYPE_STRING:
		case MYSQL_TYPE_VAR_STRING:
			((char*)buffer)[length]='\0';
			return (size_t)atoi((const char*)buffer);
		case MYSQL_TYPE_TINY_BLOB:
		case MYSQL_TYPE_BLOB:
		case MYSQL_TYPE_MEDIUM_BLOB:
		case MYSQL_TYPE_LONG_BLOB:
			return 0;
		default:
			return 0;
	}
}

CMySqlField::operator const char *()
{
	switch(bind->buffer_type)
	{
		case MYSQL_TYPE_TINY:
			snprintf(num_string,sizeof(num_string),"%d",(int)*(char*)buffer);
			return num_string;
		case MYSQL_TYPE_SHORT:
			snprintf(num_string,sizeof(num_string),"%d",(int)*(short*)buffer);
			return num_string;
		case MYSQL_TYPE_LONG:
			snprintf(num_string,sizeof(num_string),"%d",*(int*)buffer);
			return num_string;
		case MYSQL_TYPE_LONGLONG:
			/* Needs fixup for Unix */
			snprintf(num_string,sizeof(num_string),"%I64d",*(long long *)buffer);
			return num_string;
		case MYSQL_TYPE_FLOAT:
			snprintf(num_string,sizeof(num_string),"%f",*(float*)buffer);
			return num_string;
		case MYSQL_TYPE_DOUBLE:
			snprintf(num_string,sizeof(num_string),"%lf",*(double*)buffer);
			return num_string;
		case MYSQL_TYPE_TIME:
		case MYSQL_TYPE_DATE:
		case MYSQL_TYPE_DATETIME:
		case MYSQL_TYPE_TIMESTAMP:
			return "(date)";
		case MYSQL_TYPE_STRING:
		case MYSQL_TYPE_VAR_STRING:
		case MYSQL_TYPE_TINY_BLOB:
		case MYSQL_TYPE_BLOB:
		case MYSQL_TYPE_MEDIUM_BLOB:
		case MYSQL_TYPE_LONG_BLOB:
			((char*)buffer)[length]='\0';
			return (char*)buffer;
		default:
			return "(unsupported-type)";
	}
}

CMySqlField::operator const wchar_t *()
{
	const char *txt = operator const char *();
#ifdef _WIN32
	wdata.resize(strlen(txt)+1);
	MultiByteToWideChar(CP_UTF8,0,txt,-1,(wchar_t*)wdata.data(),(int)wdata.size());
	return wdata.c_str();
#else
	return L"unix-not-written-yet";
#endif
}

/**********************************************************************/

CMySqlRecordset::CMySqlRecordset()
{
	m_stmt = NULL;
	m_bind = NULL;
	m_bEof=false;
}

CMySqlRecordset::~CMySqlRecordset()
{
	Close();
}

bool CMySqlRecordset::Init()
{
	m_bEof=false;

	MYSQL_RES *meta_res = mysql_stmt_result_metadata(m_stmt);
	if (!meta_res)
		return false;

	m_fields.resize(mysql_num_fields(meta_res));
	m_bind = new MYSQL_BIND[m_fields.size()];
	memset(m_bind,0,sizeof(MYSQL_BIND)*m_fields.size());

	for(int f=0; f<(int)m_fields.size(); f++)
	{
		int len;
		MYSQL_FIELD *field = mysql_fetch_field_direct(meta_res,f);
		switch(field->type)
		{
			case MYSQL_TYPE_INT24:
				m_bind[f].buffer_type=MYSQL_TYPE_LONG;
				break;
			default:
				m_bind[f].buffer_type=field->type;
		};
		

		if(field->length>1024)
			len = 1024;
		else
			len = field->length;

		if(!IS_NUM(field->type))
			len++;

		m_bind[f].buffer_length=len;
		m_fields[f].buffer = new char[len];
		m_fields[f].bind=m_bind+f;
		m_bind[f].buffer=m_fields[f].buffer;
		m_fields[f].name=field->name;
		m_bind[f].length=&m_fields[f].length;
		m_bind[f].is_null=&m_fields[f].is_null;
	}
	mysql_stmt_bind_result(m_stmt,m_bind);
	mysql_free_result(meta_res);

	Next();
	return true;
};

bool CMySqlRecordset::Close()
{
	if(m_stmt)
		mysql_stmt_close(m_stmt);
	if(m_bind)
		delete[] m_bind;
	m_stmt=NULL;
	m_bind=NULL;
	return true;
}

bool CMySqlRecordset::Closed() const
{
	if(!m_stmt)
		return true;
	if(mysql_stmt_errno(m_stmt))
		return true;
	return false;
}

bool CMySqlRecordset::Eof() const
{
	return m_bEof;
}

bool CMySqlRecordset::Error() const
{
	if(mysql_stmt_errno(m_stmt))
		return true;
	return false;
}

bool CMySqlRecordset::Next()
{
	int ret = mysql_stmt_fetch(m_stmt);
	if(ret!=0)
		m_bEof=true;
	return !m_bEof;
}

CSqlField* CMySqlRecordset::operator[](size_t item) const
{
	if(item>=m_fields.size())
		return NULL;
	return (CSqlField*)&m_fields[item];
}

CSqlField* CMySqlRecordset::operator[](int item) const
{
	if(item<0 || item>=(int)m_fields.size())
		return NULL;
	return (CSqlField*)&m_fields[item];
}

CSqlField* CMySqlRecordset::operator[](const char *item) const
{
	for(size_t n=0; n<m_fields.size(); n++)
		if(m_fields[n].name==item)
			return (CSqlField*)&m_fields[n];
	return NULL;
}

#endif