// CompatibiltyPage.cpp : implementation file
//

#include "stdafx.h"
#include "CompatibiltyPage.h"
#include ".\compatibiltypage.h"

// CCompatibiltyPage dialog

static const struct { int val; const TCHAR *desc; const TCHAR *reg; } compat_regexp[] =
{
	{ 0, _T("Any CVS/CVSNT"),			  _T("") },
	{ 1, _T("Any CVSNT"),				  _T("^CVSNT.*") },
	{ 2, _T("CVSNT 2.0.58/CVSNT 2.5.01"), _T("^(CVSNT 2\\.0\\.58)|(CVSNT 2\\.5\\.01).*") },
	{ 3, _T("CVSNT 2.5.01"),			  _T("^CVSNT 2\\.5\\.01.*") },
	{ -1 }
};

IMPLEMENT_DYNAMIC(CCompatibiltyPage, CTooltipPropertyPage)
CCompatibiltyPage::CCompatibiltyPage()
	: CTooltipPropertyPage(CCompatibiltyPage::IDD)
{
	m_hServerKey=NULL;
}

CCompatibiltyPage::~CCompatibiltyPage()
{
	RegCloseKey(m_hServerKey);
}

void CCompatibiltyPage::DoDataExchange(CDataExchange* pDX)
{
	CTooltipPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_OLDVERSION, m_btOldVersion);
	DDX_Control(pDX, IDC_OLDCHECKOUT, m_btOldCheckout);
	DDX_Control(pDX, IDC_HIDESTATUS, m_btHideStatus);
	DDX_Control(pDX, IDC_CLIENTTYPE, m_cbClient);
	DDX_Control(pDX, IDC_ALLOWEDCLIENTS, m_cbAllowedClients);
}


BEGIN_MESSAGE_MAP(CCompatibiltyPage, CTooltipPropertyPage)
	ON_BN_CLICKED(IDC_HIDESTATUS, OnBnClickedHidestatus)
	ON_BN_CLICKED(IDC_OLDVERSION, OnBnClickedOldversion)
	ON_BN_CLICKED(IDC_OLDCHECKOUT, OnBnClickedOldcheckout)
	ON_CBN_SELENDOK(IDC_CLIENTTYPE, OnCbnSelendokClienttype)
	ON_CBN_SELENDOK(IDC_ALLOWEDCLIENTS, OnCbnSelendokAllowedclients)
END_MESSAGE_MAP()


// CCompatibiltyPage message handlers

BOOL CCompatibiltyPage::OnInitDialog()
{
	BYTE buf[_MAX_PATH*sizeof(TCHAR)];
	DWORD bufLen;
	DWORD dwType;

	CTooltipPropertyPage::OnInitDialog();

	if(!m_hServerKey && RegCreateKeyEx(HKEY_LOCAL_MACHINE,_T("Software\\CVS\\Pserver"),NULL,_T(""),REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&m_hServerKey,NULL))
		return FALSE;

	m_cbClient.ResetContent();
	m_cbClient.SetItemData(m_cbClient.AddString(_T("Non-cvsnt clients")),0);
	m_cbClient.SetItemData(m_cbClient.AddString(_T("CVSNT compatible clients")),1);
	m_cbClient.SetCurSel(0);

	m_cbAllowedClients.ResetContent();
	for(size_t n=0; compat_regexp[n].val>=0; n++)
	{
		m_cbAllowedClients.SetItemData(m_cbAllowedClients.AddString(compat_regexp[n].desc),compat_regexp[n].val);
	}

	bufLen=sizeof(buf);
	if(RegQueryValueEx(m_hServerKey,_T("AllowedClients"),NULL,&dwType,buf,&bufLen))
	{
		m_cbAllowedClients.SetCurSel(0);
	}
	else
	{
		for(size_t n=0; compat_regexp[n].val>=0; n++)
		{
			if(!_tcscmp((const TCHAR*)buf,compat_regexp[n].reg))
				m_cbAllowedClients.SetCurSel(m_cbAllowedClients.FindStringExact(-1,compat_regexp[n].desc));
		}
	}

	CompatibilityData.resize(2);

	if(QueryDword(_T("FakeUnixCvs"),0))
	{
		CompatibilityData[0].OldVersion=1;
		CompatibilityData[0].OldCheckout=1;
		CompatibilityData[0].HideStatus=1;
		RegDeleteValue(m_hServerKey,_T("FakeUnixCvs"));
		GetParent()->PostMessage(PSM_CHANGED, (WPARAM)m_hWnd); /* SetModified happens too early */
	}
	else
	{
		CompatibilityData[0].HideStatus=QueryDword(_T("Compat0_OldVersion"),0);
		CompatibilityData[0].OldCheckout=QueryDword(_T("Compat0_OldCheckout"),0);
		CompatibilityData[0].OldVersion=QueryDword(_T("Compat0_HideStatus"),0);
	}

	CompatibilityData[1].HideStatus=QueryDword(_T("Compat1_OldVersion"),0);
	CompatibilityData[1].OldCheckout=QueryDword(_T("Compat1_OldCheckout"),0);
	CompatibilityData[1].OldVersion=QueryDword(_T("Compat1_HideStatus"),0);

	FillClientType();

	return TRUE;
}

DWORD CCompatibiltyPage::QueryDword(LPCTSTR szKey, DWORD dwDefault)
{
	BYTE buf[64];
	DWORD bufLen=sizeof(buf);
	DWORD dwType;
	if(RegQueryValueEx(m_hServerKey,szKey,NULL,&dwType,buf,&bufLen))
		return dwDefault;
	if(dwType!=REG_DWORD)
		return dwDefault;
	return *(DWORD*)buf;
}

void CCompatibiltyPage::SetDword(LPCTSTR szKey, DWORD dwVal)
{
	if(RegSetValueEx(m_hServerKey, szKey,NULL,REG_DWORD,(BYTE*)&dwVal,sizeof(DWORD)))
		AfxMessageBox(_T("RegSetValueEx failed"),MB_ICONSTOP);
}

BOOL CCompatibiltyPage::OnApply()
{
	for(size_t n=0; n<CompatibilityData.size(); n++)
	{
		CString f;

		f.Format(_T("Compat%d_OldVersion"),n);
		SetDword(f,CompatibilityData[n].OldVersion);
		f.Format(_T("Compat%d_OldCheckout"),n);
		SetDword(f,CompatibilityData[n].OldCheckout);
		f.Format(_T("Compat%d_HideStatus"),n);
		SetDword(f,CompatibilityData[n].HideStatus);
	}

	const TCHAR *allowed = _T("");
	int s = m_cbAllowedClients.GetCurSel();
	if(s!=-1)
	{
		s=m_cbAllowedClients.GetItemData(s);
		for(size_t n=0; compat_regexp[n].val>=0; n++)
		{
			if(compat_regexp[n].val==s)
			{
				allowed=compat_regexp[n].reg;
				break;
			}
		}


		if(RegSetValueEx(m_hServerKey,_T("AllowedClients"),NULL,REG_SZ,(BYTE*)allowed,(_tcslen(allowed)+1)*sizeof(TCHAR)))
			AfxMessageBox(_T("RegSetValueEx failed"),MB_ICONSTOP);
	}

	return CTooltipPropertyPage::OnApply();
}

void CCompatibiltyPage::OnBnClickedOldversion()
{
	SetModified();
	CompatibilityData[m_cbClient.GetCurSel()].OldVersion=m_btOldVersion.GetCheck();
}

void CCompatibiltyPage::OnBnClickedOldcheckout()
{
	SetModified();
	CompatibilityData[m_cbClient.GetCurSel()].OldCheckout=m_btOldCheckout.GetCheck();
}

void CCompatibiltyPage::OnBnClickedHidestatus()
{
	SetModified();
	CompatibilityData[m_cbClient.GetCurSel()].HideStatus=m_btHideStatus.GetCheck();
}

void CCompatibiltyPage::OnCbnSelendokClienttype()
{
	FillClientType();
}

void CCompatibiltyPage::FillClientType()
{
	int client = m_cbClient.GetCurSel();

	m_btOldVersion.SetCheck(CompatibilityData[client].OldVersion?1:0);
	m_btOldCheckout.SetCheck(CompatibilityData[client].OldCheckout?1:0);
	m_btHideStatus.SetCheck(CompatibilityData[client].HideStatus?1:0);
}


void CCompatibiltyPage::OnCbnSelendokAllowedclients()
{
	SetModified();
}
