/*	CVS info library interface
    Copyright (C) 2004 Tony Hoyle and March-Hare Software Ltd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/* info DLLs need to export one function:

	library_callback *GetCvsInfo(int version)

	Note that the v4 interface is different to its predecessors and will require some
	rewriting to upgrade.
*/

#ifndef INFOLIB__H
#define INFOLIB__H

#ifdef __cplusplus
extern "C" {
#endif

/* Structure passed into loginfo function */
typedef struct change_info_t
{
	const char *filename;
	const char *rev_new;
	const char *rev_old;
	char type;
	const char *tag;
	const char *bugid;
} change_info;

/* Main callback structure.  This should be returned as an intitialised structure
   by the GetCvsInfo structure. */
typedef struct library_callback_t
{
	int (*init)(struct library_callback_t* cb, const char *command, const char *date, const char *hostname, const char *username, const char *virtual_repository, const char *physical_repository, const char *sessionid, const char *editor, int count_uservar, const char **uservar, const char **userval, const char *client_version);
	int (*close)(struct library_callback_t* cb);
	int (*pretag)(struct library_callback_t* cb, const char *message, const char *directory, int name_list_count, const char **name_list, const char **version_list, char tag_type, const char *action, const char *tag);
	int (*verifymsg)(struct library_callback_t* cb, const char *directory, const char *filename);
	int (*loginfo)(struct library_callback_t* cb, const char *message, const char *status, const char *directory, int change_list_count, change_info_t *change_list);
	int (*history)(struct library_callback_t* cb, char type, const char *workdir, const char *revs, const char *name);
	int (*notify)(struct library_callback_t* cb, const char *message, const char *bugid, const char *directory, const char *notify_user, const char *tag, const char *type, const char *file);
	int (*precommit)(struct library_callback_t* cb, int name_list_count, const char **name_list, const char *message, const char *directory);
	int (*postcommit)(struct library_callback_t* cb, const char *directory);
	int (*precommand)(struct library_callback_t* cb, int argc, const char **argv);
	int (*postcommand)(struct library_callback_t* cb, const char *directory);
	int (*premodule)(struct library_callback_t* cb, const char *module);
	int (*postmodule)(struct library_callback_t* cb, const char *module);
	int (*get_template)(struct library_callback_t *cb, const char *directory, const char **template_ptr);

	void *_reserved;
} library_callback;

typedef library_callback *(*CVSINFOPROC)(int version);

#define CVSINFO_VERSION 0x400

#ifdef __cplusplus
}
#endif

#endif
