/* CVS info library interface

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.  */


#include "cvs.h"
#define MAIN_CVS
#include <config.h>

#include <map>
#include <string>

#include "infolib.h"
#include "../protocols/library.h"

static infolib_list_t infolib_list;

const infolib_list_t& get_infolib_list()
{
	return infolib_list;
}

library_callback *open_infolibrary(const char *filter)
{
	library_callback *cb = NULL;
	if(infolib_list.find(filter)!=infolib_list.end())
	{
		cb = infolib_list[filter];
		return cb;
	}

#ifdef USE_SHARED_LIBS
	library_handle_t handle;
	CVSINFOPROC pCvsInfo = (CVSINFOPROC)library_load(filter,"GetCvsInfo",handle);
	if(!pCvsInfo)
	  return NULL;

	cb = pCvsInfo(CVSINFO_VERSION);
	if(!cb)
	{
		TRACE(3,"GetCvsInfo() function failed in '%s'",filter);
		library_unload(handle);
		return NULL;
	}
	cb->_reserved = handle;
#else
	error(0,0,"Shared library '%s' not supported in this build",filter);
	return NULL;
#endif
	infolib_list[filter]=cb;
	if(cb->init)
	{
		int count_uservar;
		const char **uservar,**userval;
		init_uservar(&count_uservar,&uservar,&userval);
		cb->init(cb,server_active?server_command_name:command_name,global_session_time,remote_host_name?remote_host_name:hostname,CVS_Username,current_parsed_root->unparsed_directory,current_parsed_root->directory,global_session_id,Editor,count_uservar,uservar,userval,serv_client_version);
	}
	return cb;
}

int close_infolibrary(library_callback *cb)
{
	return 0;
};

int shutdown_infolib()
{
	std::map<std::string, library_callback *>::const_iterator i;
	for(i=infolib_list.begin();i!=infolib_list.end();i++)
	{
		if(!i->second)
			continue;
		TRACE(3,"Unloading %s",i->first.c_str());
		if(i->second->close)
			i->second->close(i->second);
#ifdef USE_SHARED_LIBS
		library_unload((library_handle_t)i->second->_reserved);
#endif
	}
	infolib_list.clear();
	return 0;
}
