/* CVS info library interface

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.  */

#define WIN32_LEAN_AND_MEAN
#define _WIN32_WINNT 0x0400
#include <windows.h>

#include "cvs.h"

#include "infolib.h"

#ifndef CVS95

#include <comdef.h>
#include <objbase.h>

#include "cvscom_h.h"

struct InfoStruct
{
	ICvsInfo4 *i4;

	std::vector<void*> to_free;
};

class _ubstr_t
{
public:
	_ubstr_t(const char *str)
	{
		if(str)
		{
			int l = strlen(str)*2+2;
			wchar_t *wstr = (wchar_t*)xmalloc(l);
			MultiByteToWideChar(win32_global_codepage,0,str,-1,wstr,l);
			bStr = SysAllocString(wstr);
			xfree(wstr);
		}
		else
			bStr=NULL;
	}
	~_ubstr_t()
	{
		if(bStr)
			SysFreeString(bStr);
	}
	operator BSTR() { return bStr; }
	BSTR Detach() { BSTR b = bStr; bStr=NULL; return b; }
protected:
	BSTR bStr;
};

#endif

static infolib_list_t infolib_list;

const infolib_list_t& get_infolib_list()
{
	return infolib_list;
}

#ifndef CVS95
int COM_init(struct library_callback_t* cb, const char *command, const char *date, const char *hostname, const char *username, const char *virtual_repository, const char *physical_repository, const char *sessionid, const char *editor, int count_uservar, const char **uservar, const char **userval, const char *client_version)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	int ret;
	SAFEARRAY *sa_uservar = SafeArrayCreateVector(VT_BSTR, 0, count_uservar);
	SAFEARRAY *sa_userval = SafeArrayCreateVector(VT_BSTR, 0, count_uservar);
	for(long n=0; n<count_uservar; n++)
	{
		SafeArrayPutElement(sa_uservar, &n, _ubstr_t(uservar[n]).Detach());
		SafeArrayPutElement(sa_userval, &n, _ubstr_t(userval[n]).Detach());
	}
	ret = i->i4->init(_ubstr_t(command),_ubstr_t(date),_ubstr_t(hostname),_ubstr_t(virtual_repository),_ubstr_t(physical_repository),_ubstr_t(sessionid), _ubstr_t(editor), sa_uservar, sa_userval, _ubstr_t(client_version));
	SafeArrayDestroy(sa_uservar);
	SafeArrayDestroy(sa_userval);
	return ret;
}

int COM_pretag(struct library_callback_t* cb, const char *message, const char *directory, int name_list_count, const char **name_list, const char **version_list, char tag_type, const char *action, const char *tag)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	int ret;
	SAFEARRAY *sa_name_list = SafeArrayCreateVector(VT_BSTR, 0, name_list_count);
	SAFEARRAY *sa_version_list = SafeArrayCreateVector(VT_BSTR, 0, name_list_count);
	for(long n=0; n<name_list_count; n++)
	{
		SafeArrayPutElement(sa_name_list, &n, _ubstr_t(name_list[n]).Detach());
		SafeArrayPutElement(sa_version_list, &n, _ubstr_t(version_list[n]).Detach());
	}
	ret = i->i4->pretag(_ubstr_t(message),_ubstr_t(directory),sa_name_list,sa_version_list, tag_type,_ubstr_t(action),_ubstr_t(tag));
	SafeArrayDestroy(sa_name_list);
	SafeArrayDestroy(sa_version_list);
	return ret;
}

int COM_verifymsg(struct library_callback_t* cb, const char *directory, const char *filename)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	return i->i4->verifymsg(_ubstr_t(directory),_ubstr_t(filename));
}

int COM_loginfo(struct library_callback_t* cb, const char *message, const char *status, const char *directory, int change_list_count, change_info_t *change_list)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	int ret;
	ChangeInfoStruct *pData;
	IRecordInfo *pRI;
	HRESULT hr;
	hr = GetRecordInfoFromGuids(LIBID_CVSNT, 1, 0, 0x409, __uuidof(ChangeInfoStruct), &pRI);
	if(FAILED(hr))
	{
		error(1,0,"GetRecordInfoFromGuids returned 0x%x\n",hr);
	}
	SAFEARRAY *sa_change_list = SafeArrayCreateVectorEx(VT_RECORD, 0, change_list_count,pRI);
	pRI->Release();
	hr = SafeArrayAccessData(sa_change_list, (void**)&pData);
	if(FAILED(hr))
	{
		error(1,0,"SafeArrayAccessData returned 0x%x\n",hr);
	}
	for(long n=0; n<change_list_count; n++)
	{
		pData[n].filename=_ubstr_t(change_list[n].filename).Detach();
		pData[n].rev_new=_ubstr_t(change_list[n].rev_new).Detach();
		pData[n].rev_old=_ubstr_t(change_list[n].rev_old).Detach();
		pData[n].tag=_ubstr_t(change_list[n].tag).Detach();
		pData[n].type=change_list[n].type;
		pData[n].bugid=_ubstr_t(change_list[n].bugid).Detach();
	}
	hr = SafeArrayUnaccessData(sa_change_list);
	ret = i->i4->loginfo(_ubstr_t(message),_ubstr_t(status),_ubstr_t(directory),sa_change_list);
	SafeArrayDestroy(sa_change_list);
	return ret;
}

int COM_history(struct library_callback_t* cb, char type, const char *workdir, const char *revs, const char *name)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	return i->i4->history(type,_ubstr_t(workdir),_ubstr_t(revs),_ubstr_t(name));
}

int COM_notify(struct library_callback_t* cb, const char *message, const char *bugid, const char *directory, const char *notify_user, const char *tag, const char *type, const char *file)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	return i->i4->notify(_ubstr_t(message),_ubstr_t(bugid),_ubstr_t(directory),_ubstr_t(notify_user),_ubstr_t(tag),_ubstr_t(type),_ubstr_t(file));
}

int COM_precommit(struct library_callback_t* cb, int name_list_count, const char **name_list, const char *message, const char *directory)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	int ret;
	SAFEARRAY *sa_name_list = SafeArrayCreateVector(VT_BSTR, 0, name_list_count);
	for(long n=0; n<name_list_count; n++)
		SafeArrayPutElement(sa_name_list, &n, (BSTR)_ubstr_t(name_list[n]).Detach());
	ret = i->i4->precommit(sa_name_list,_ubstr_t(message),_ubstr_t(directory));
	SafeArrayDestroy(sa_name_list);
	return ret;
}

int COM_precommand(struct library_callback_t* cb, int argc, const char **argv)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	int ret;
	SAFEARRAY *sa_precommand_list = SafeArrayCreateVector(VT_BSTR, 0, argc);
	for(long n=0; n<argc; n++)
		SafeArrayPutElement(sa_precommand_list, &n, (BSTR)_ubstr_t(argv[n]).Detach());
	ret = i->i4->precommand(sa_precommand_list);
	SafeArrayDestroy(sa_precommand_list);
	return ret;
}

int COM_postcommand(struct library_callback_t* cb, const char *directory)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	return i->i4->postcommand(_ubstr_t(directory));
}

int COM_postcommit(struct library_callback_t* cb, const char *directory)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	return i->i4->postcommit(_ubstr_t(directory));
}

int COM_premodule(struct library_callback_t* cb, const char *module)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	return i->i4->premodule(_ubstr_t(module));
}

int COM_postmodule(struct library_callback_t* cb, const char *module)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	return i->i4->postmodule(_ubstr_t(module));
}

int COM_get_template(struct library_callback_t *cb, const char *directory, const char **template_ptr)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;
	int ret = 0;
	BSTR bTemplate;
	ret = i->i4->get_template(_ubstr_t(directory),&bTemplate);
	if(!ret)
	{
		wchar_t *wstr = bTemplate;
		int l = wcslen(wstr)*4;
		char *str = (char*)xmalloc(l); /* widechar to UTF8 could get quite long... */
		i->to_free.push_back(str);
		WideCharToMultiByte(win32_global_codepage,0,wstr,-1,str,l,"?",NULL);
		*template_ptr = str;
	}
	return ret;
}

int COM_close(struct library_callback_t *cb)
{
	InfoStruct *i = (InfoStruct*)cb->_reserved;

	i->i4->Release();

	for(size_t n=0; n<i->to_free.size(); n++)
		xfree(i->to_free[n]);

	delete i;
	delete cb;
	return 0;
}

#endif

library_callback *open_infolibrary(const char *library)
{
	library_callback *cb = NULL;

	/* We do this here so it's done before the CRT in the DLL/COM object initialises */
	if(server_io_socket)
	{
		SetStdHandle(STD_INPUT_HANDLE,(HANDLE)_get_osfhandle(server_io_socket));
		SetStdHandle(STD_OUTPUT_HANDLE,(HANDLE)_get_osfhandle(server_io_socket));
		SetStdHandle(STD_ERROR_HANDLE,(HANDLE)_get_osfhandle(server_io_socket));
	}

	cb = infolib_list[library];
	if(cb)
		return cb;

#ifndef CVS95
	if(library[0]=='{')
	{
		/* COM filter */
		CLSID id;
		wchar_t str[128];
		char *p = strchr(library,'}');
		if(!p)
			return 0;

		MultiByteToWideChar(win32_global_codepage,0,library,(p-library)+1,str,sizeof(str)/sizeof(wchar_t));
		str[(p-library)+1]='\0';
		HRESULT hr = CLSIDFromString(str,&id);
		if(hr!=NOERROR)
		{
			str[wcslen(str)-1]='\0';
			hr = CLSIDFromProgID(str+1,&id);
			if(hr!=NOERROR)
				return NULL;
		}
		ICvsInfo4 *i4 = NULL;
		hr = CoCreateInstance(id,NULL,CLSCTX_ALL,IID_ICvsInfo4,(void**)&i4);
		if(FAILED(hr))
		{
			if(FAILED(hr))
			{
				TRACE(3,"Failed to read COM reference to {%S} - must support at least revision 4",str);
				return NULL;
			}
		}

		cb =  new library_callback;
		InfoStruct *is = new InfoStruct;
		is->i4 = i4;
		cb->_reserved=(void*)is;
		cb->init=COM_init;
		cb->close=COM_close;
		cb->history=COM_history;
		cb->notify=COM_notify;
		cb->precommand=COM_precommand;
		cb->postcommand=COM_postcommand;
		cb->premodule=COM_premodule;
		cb->postmodule=COM_postmodule;
		cb->precommit=COM_precommit;
		cb->postcommit=COM_postcommit;
		cb->pretag=COM_pretag;
		cb->verifymsg=COM_verifymsg;
		cb->loginfo=COM_loginfo;
		cb->get_template=COM_get_template;
	}
	else
#endif
	{
		HMODULE hLib = LoadLibraryA(library);
		if(!hLib)
			return NULL;
		CVSINFOPROC pCvsInfo = (CVSINFOPROC)GetProcAddress(hLib,"GetCvsInfo");
		if(!pCvsInfo)
			return NULL;
		cb = pCvsInfo(CVSINFO_VERSION);
		if(!cb)
		{
			FreeLibrary(hLib);
		}
		cb->_reserved = hLib;
	}
	if(cb)
	{
		infolib_list[library]=cb;
		if(cb->init)
		{
			int count_uservar;
			const char **uservar,**userval;
			init_uservar(&count_uservar,&uservar,&userval);
			cb->init(cb,server_active?server_command_name:command_name,global_session_time,remote_host_name?remote_host_name:hostname,CVS_Username,current_parsed_root->unparsed_directory,current_parsed_root->directory,global_session_id,Editor,count_uservar,uservar,userval,serv_client_version);
		}
	}
	return cb;
}

int close_infolibrary(library_callback *cb)
{
	return 0;
};

int shutdown_infolib()
{
	std::map<std::string, library_callback *>::const_iterator i;
	for(i=infolib_list.begin();i!=infolib_list.end();i++)
	{
		if(i->second)
		{
			TRACE(3,"Unloading %s",i->first.c_str());
			if(i->second->close)
				i->second->close(i->second);
			FreeLibrary((HMODULE)i->second->_reserved);
		}
	}
	infolib_list.clear();
	return 0;
}
