/*
	CVSNT Generic API
    Copyright (C) 2004 Tony Hoyle and March-Hare Software Ltd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef XMLAPI__H
#define XMLAPI__H

#include <map>
#include <vector>
#include <string>

#include <expat.h>

#include "../cvsapi/cvsapi.h"

class CXmlTree;

class CXmlNode
{
	friend class CXmlTree;
public:
	typedef std::vector<cvs::smartptr<CXmlNode*> > ChildArray_t;

	enum XmlTypeEnum
	{
		XmlTypeNode,
		XmlTypeAttribute
	};

	CXmlNode(CXmlTree* tree) { m_tree = tree; XmlType=XmlTypeNode; Parent=NULL; StartLineNumber=EndLineNumber=0; keyNum=0; sorted=false; }
	CXmlNode(CXmlTree* tree, XmlTypeEnum Type, const char *name, const char *value) { m_tree = tree; XmlType=Type; Parent=NULL; this->name=name; if(value) text=text; StartLineNumber=EndLineNumber=0; keyNum=0; sorted=false; }
	CXmlNode(const CXmlNode& other);
	~CXmlNode() { }
	CXmlNode *NewAttribute(const char *name, const char *value) { return _New(XmlTypeAttribute, name, value); }
	CXmlNode *NewNode(const char *name, const char *value) { return _New(XmlTypeNode, name, value); }
	void SetValue(const char *value) { text = value?value:""; }
	void AppendValue(const char *value, size_t len) { if(value) text.append(value,len); }
	const char *GetValue() const { return text.c_str(); }
	const char *GetName() const { return name.c_str(); }
	XmlTypeEnum getType() const { return XmlType; }
	CXmlNode *Lookup(const char *path, bool autoCreate = false);
	CXmlNode *Next();
	CXmlNode *Previous();
	CXmlNode *getParent() const { return this->Parent; }
	bool Delete(CXmlNode *child);
	bool BatchDelete();
	bool Prune();
	CXmlNode *Copy();
	bool Paste(CXmlNode *from);
	size_t size() const { return Children.size(); }
	ChildArray_t::iterator begin() { return Children.begin(); }
	ChildArray_t::iterator end() { return Children.end(); }
	int getStartLine() const { return StartLineNumber; }
	int getEndLine() const { return EndLineNumber; }

	bool WriteXmlFile(FILE *file) const;
	bool Sort() { return SortMe(); }

protected:
	std::string name;
	std::string text;
	int keyNum;
	bool sorted;
	ChildArray_t Children;
	CXmlNode *Parent;
	XmlTypeEnum XmlType;
	int StartLineNumber,EndLineNumber;
	CXmlTree *m_tree;

	CXmlNode *_New(XmlTypeEnum Type, const char *name, const char *value);
	bool WriteXmlNode(FILE *file, int indent) const;
	ChildArray_t::iterator FindIterator(CXmlNode *node);
	int cmp(CXmlNode* b);
	static bool sortPred(ChildArray_t::value_type a, ChildArray_t::value_type b);
	bool SortMe();
};

class CXmlTree
{
	friend class CXmlNode;

public:
	CXmlTree();
	virtual ~CXmlTree();

	CXmlNode *ReadXmlFile(FILE* file);
	CXmlNode *ReadXmlFile(FILE* file, std::vector<cvs::string>& ignore_tag);

protected:
	CCodepage m_cp;
	CXmlNode* m_lastNode;

	static void startElement(void *userData, const char *name, const char **atts);
	static void endElement(void *userData, const char *name);
	static void charData(void *userData, const char * s, int len);
	static int getEncoding(void *userData, const XML_Char *name, XML_Encoding *info);
};

#endif
